"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OPCUAWsSecureServerEndPoint = exports.OPCUAWsServerEndPoint = exports.TransportType = void 0;
// import * as chalk from 'chalk';
const chalk = require("chalk");
const node_opcua_1 = require("node-opcua");
const node_opcua_crypto_1 = require("node-opcua-crypto");
const ws = require("ws");
const https = require("https");
const websocket_socket_wrapper_1 = require("./websocket-socket-wrapper");
const node_opcua_debug_1 = require("node-opcua-debug");
const debugLog = node_opcua_debug_1.make_debugLog(__filename);
function extractSocketData(socket, reason) {
    const { bytesRead, bytesWritten, remoteAddress, remoteFamily, remotePort, localAddress, localPort, } = socket;
    return {
        bytesRead,
        bytesWritten,
        localAddress,
        localPort,
        remoteAddress,
        remoteFamily,
        remotePort,
        timestamp: new Date(),
    };
}
var TransportType;
(function (TransportType) {
    TransportType[TransportType["TCP"] = 0] = "TCP";
    TransportType[TransportType["WEBSOCKET"] = 1] = "WEBSOCKET";
    TransportType[TransportType["WEBSOCKET_SECURE"] = 2] = "WEBSOCKET_SECURE";
})(TransportType = exports.TransportType || (exports.TransportType = {}));
// @ts-ignore // ignore overriding the type of _server
class OPCUAWsServerEndPoint extends node_opcua_1.OPCUAServerEndPoint {
    constructor() {
        super(...arguments);
        this._started = false;
        this._channels = {};
    }
    _setup_server() {
        // nothing to do
    }
    _dump_statistics() {
        const self = this;
        debugLog(chalk.cyan("CONCURRENT CONNECTION = "), self._server.clients.size);
        debugLog(chalk.cyan("MAX CONNECTIONS = "), self._server.getMaxListeners());
    }
    protocolName() {
        return "opc.ws";
    }
    listen(callback) {
        node_opcua_1.assert(typeof callback === "function");
        node_opcua_1.assert(!this._started, "OPCUAWSServerEndPoint is already listening");
        node_opcua_1.assert(!this._server);
        this._server = this.createWSServer();
        this._server.setMaxListeners(this.maxConnections + 1); // plus one extra
        // like net.Server
        this._server.maxConnections = this._server.getMaxListeners();
        // @ts-ignore private access
        this._listen_callback = callback;
        this._server
            .on("connection", (socket, request) => {
            /*
                if (doDebug) {
                    this._dump_statistics();
                    debugLog("server connected with : " +
                      request.socket.remoteAddress + ":" + request.socket.remotePort);
                }
                */
            // @ts-ignore private access
            this._on_client_connection(new websocket_socket_wrapper_1.WebSocketSocketWrapper(socket), request);
        })
            .on("close", () => {
            debugLog("server closed : all connections have ended");
        })
            .on("error", (err) => {
            debugLog(chalk.red.bold(" error") + " port = " + this.port, err);
            this._started = false;
            // @ts-ignore private access
            this._end_listen(err);
        })
            .on("listening", () => {
            debugLog("server is listening");
            // @ts-ignore private access
            this._end_listen();
        });
        this._started = true;
    }
    createWSServer() {
        const wsserver = new ws.Server({
            port: this.port,
            verifyClient: (info) => this._verifyClient(info),
        });
        return wsserver;
    }
    _verifyClient(info) {
        if (!this._started) {
            debugLog(chalk.bgWhite.cyan("OPCUAWsServerEndPoint#_on_client_connection " +
                "SERVER END POINT IS PROBABLY SHUTTING DOWN !!! - Connection is refused"));
            return false;
        }
        const nbConnections = Object.keys(this._channels).length;
        debugLog(" nbConnections ", nbConnections, " self._server.maxListeners", this._server.getMaxListeners(), this.maxConnections);
        if (nbConnections >= this.maxConnections) {
            console.log(chalk.bgWhite.cyan("OPCUAServerEndPoint#_on_client_connection " +
                "The maximum number of connection has been reached - Connection is refused"));
            const reason = "maxConnections reached (" + this.maxConnections + ")";
            const socketData = extractSocketData(info.req.socket, reason);
            this.emit("connectionRefused", socketData);
            return false;
        }
        return true;
    }
    addEndpointDescription(securityMode, securityPolicy, options) {
        super.addEndpointDescription(securityMode, securityPolicy, options);
        const endpoints = this.endpointDescriptions();
        const ep = endpoints[endpoints.length - 1];
        if (ep.endpointUrl) {
            const endpointUrl = ep.endpointUrl.replace("opc.tcp", this.protocolName());
            ep.__defineGetter__("endpointUrl", () => endpointUrl);
        }
    }
}
exports.OPCUAWsServerEndPoint = OPCUAWsServerEndPoint;
class OPCUAWsSecureServerEndPoint extends OPCUAWsServerEndPoint {
    createWSServer() {
        const pemCert = node_opcua_crypto_1.toPem(this.getCertificate(), "CERTIFICATE");
        const httpServer = https.createServer({
            cert: pemCert,
            key: this.getPrivateKey(),
        });
        const wsserver = new ws.Server({
            // port: this.port,
            verifyClient: (info) => this._verifyClient(info),
            server: httpServer,
        });
        httpServer.listen(this.port);
        return wsserver;
    }
    protocolName() {
        return "opc.wss";
    }
}
exports.OPCUAWsSecureServerEndPoint = OPCUAWsSecureServerEndPoint;
//# sourceMappingURL=server-endpoint.js.map